//
//  ContentView.swift
//  TCOnboardingTemplate
//
//  Created by Jean-Marc Boullianne on 6/12/20.
//  Copyright © 2020 TrailingClosure. All rights reserved.
//

import SwiftUI

struct OnboardingView: View {
    
    @Binding var finished: Bool
    
    @State var index: Int = 0
    @State var count: Int = 3
    
    // MODIFY THESE VARIABLES TO CHANGE THE ONBOARDING SCREEN
    var imageNames: [String] = ["image 1", "image 2", "image 3"]
    var titles: [String] = ["Loem Ipsum", "Loem Ipsum", "Loem Ipsum"]
    var descriptions: [String] = ["Lorem ipsum dolor sit amet. Consecteteuer adipiscing elit.", "Lorem ipsum dolor sit amet. Consecteteuer adipiscing elit.", "Lorem ipsum dolor sit amet. Consecteteuer adipiscing elit."]
    
    var blobStartColor: Color = Color(red: 220/255, green: 217/255, blue: 239/255)
    var blobEndColor: Color = Color(red: 104/255, green: 83/255, blue: 235/255)
    
    var pageControlColor: Color = Color(red: 104/255, green: 83/255, blue: 235/255)
    
    var titleColor: Color = Color(red: 18/255, green: 31/255, blue: 71/255)
    var detailColor: Color = Color(red: 18/255, green: 31/255, blue: 71/255)
    
    var buttonColor: Color = Color(red: 104/255, green: 83/255, blue: 235/255)
    
    @State private var dragOffset: CGFloat = 0
    
    var body: some View {
        GeometryReader { geometry in
            ZStack {
                
                // Background Blobs
                VStack {
                    Blob()
                        .fill(LinearGradient(gradient: Gradient(colors: [self.blobStartColor, self.blobEndColor]), startPoint: UnitPoint(x: 0, y: 0), endPoint: UnitPoint(x: 1, y: 1)))
                        .frame(width: geometry.size.width, height: geometry.size.height/3.0, alignment: .center)
                        .offset(x: getBlobXOffset(pageIndex: self.index, proxy: geometry), y: -70)
                    Spacer()
                    Blob()
                        .fill(LinearGradient(gradient: Gradient(colors: [self.blobStartColor, self.blobEndColor]), startPoint: UnitPoint(x: 0, y: 0), endPoint: UnitPoint(x: 1, y: 1)))
                        .frame(width: geometry.size.width, height: geometry.size.height/3.0, alignment: .center)
                        .offset(x: getBlobXOffset(pageIndex: self.index, proxy: geometry), y: -70)
                        .rotationEffect(Angle(degrees: 180))
                }
                .offset(x: getBlobStackOffset(proxy: geometry))
                .edgesIgnoringSafeArea(.vertical)
                .shadow(color: self.blobStartColor.opacity(0.5), radius: 10, x: 0, y: 0)
                
                VStack {
                    HStack(alignment: .center, spacing: 0){
                            ForEach(0..<self.count) { i in
                            
                            VStack(alignment: .center, spacing: 20) {
                                Image(self.imageNames[i])
                                    .resizable()
                                    .scaledToFill()
                                    .foregroundColor(self.titleColor)
                                    .frame(width: 200, height: 200, alignment: .center)
                                    .shadow(color: Color.black.opacity(0.2), radius: 15, x: 0, y: 0)
                                Text(self.titles[i])
                                    .font(.headline)
                                    .fontWeight(.bold)
                                    .foregroundColor(self.detailColor)
                                Text(self.descriptions[i])
                                    .font(.body)
                                    .fontWeight(.semibold)
                                    .fixedSize(horizontal: false, vertical: true)
                                    .multilineTextAlignment(.center)
                                    .foregroundColor(Color(red: 18/255, green: 31/255, blue: 71/255).opacity(0.4))
                            }.frame(width: geometry.size.width, height: nil, alignment: .center)
                             
                            }
                    }.offset(x: getItemOffset(pageIndex: self.index, proxy: geometry), y: 0)
                    
                    // Page control and get started
                    if self.index == self.count - 1 {
                        Button(action: {
                            self.finished = true
                        }) {
                            self.buttonColor
                            .frame(width: 250, height: 50, alignment: .center)
                            .cornerRadius(25)
                            .overlay(
                                Text("Get Started")
                                    .font(.headline)
                                    .fontWeight(.bold)
                                    .foregroundColor(.white)
                            )
                                .shadow(color: self.buttonColor.opacity(0.1), radius: 15, x: 0, y: 0)
                            .shadow(color: Color.black.opacity(0.15), radius: 15, x: 0, y: 0)
                            .padding(.top, 80)
                            
                        }.buttonStyle(PlainButtonStyle())
                        .offset(x: getBlobStackOffset(proxy: geometry))
                        
                    } else {
                        PageControl(index: self.index, count: self.count, color: self.pageControlColor)
                            .frame(width: nil, height: 50, alignment: .center)
                            .padding(.top, 80)
                            .offset(x: getBlobStackOffset(proxy: geometry))
                        
                    }
                }
            }.gesture(DragGesture()
                .onChanged({ value in
                    let dx = (value.startLocation.x - value.location.x) * -1
                    self.dragOffset = dx
                })
                .onEnded({ value in
                    let rawPosition = (geometry.size.width * CGFloat(self.index - 1) * -1) +  self.dragOffset
                    
                    let fullWidth = geometry.size.width * CGFloat(self.count)
                    
                    let finalPosition = (fullWidth / 2.0) - rawPosition
                    
                    var newIndex = Int(finalPosition / (fullWidth / CGFloat(self.count) ) )
                    
                    if newIndex >= self.count {
                        newIndex = self.count - 1
                    }
                    if newIndex < 0 {
                        newIndex = 0
                    }
                    
                    withAnimation {
                        self.dragOffset = 0
                        self.index = newIndex
                    }
                    
                    print(rawPosition, fullWidth, newIndex)
                })
            )
        }
        
    }
    
    
    // Must account for changes in iOS 14.0 GeometryReader alignment changes
    func getItemOffset(pageIndex: Int, proxy: GeometryProxy) -> CGFloat {
        if #available(iOS 14.0, *) {
            return (proxy.size.width * CGFloat(self.index) * -1) +  self.dragOffset
        } else {
            return (proxy.size.width * CGFloat(self.index - 1) * -1) +  self.dragOffset
        }
    }
    
    func getBlobXOffset(pageIndex: Int, proxy: GeometryProxy) -> CGFloat {
        return (proxy.size.width/3.0) + ((proxy.size.width * CGFloat(self.index - 1) * -1) +  self.dragOffset) * 0.1
    }
    
    func getBlobStackOffset(proxy: GeometryProxy) -> CGFloat {
        if #available(iOS 14.0, *) {
            return -1 * proxy.size.width
        } else {
            return 0
        }
    }
}

struct OnboardingView_Previews: PreviewProvider {
    @State static var finished: Bool = false
    static var previews: some View {
        OnboardingView(finished: $finished)
    }
}
